#!/usr/bin/python3
# -*- coding: utf-8 -*

# Programme de Photomaton avec possibilité de 1 photo ou 4 photos/page ou 8 photos/page
# Script TRES fortement basé sur un travail trouvé sur Internet et appelé
# boothy - Photobooth application for Raspberry Pi
# developed by Kenneth Centurion
#
# Cette version a été réalisée par Alban TREVILLY en juin 2019 sur Raspberry Pi modèle 3B+
# Travail personnel effectué par plaisir et pour rendre service, notamment à une personne qui se reconnaîtra
#
# Cette application :
# - Prendra soit 1 photo, soit 4 photos, soit 8 photos avec un délai de 5 secondes (adaptez "PHOTO_DELAY" pour le délai).
# - Superposera un texte à l'écran avec un compte à rebours.
# - Si choix de 4 photos, fusionnera les 4 photos (ou 3 + une quatrième image "4logo.png") dans une grille.
# - Si choix de 8 photos, fusionnera les 8 photos (ou 7 + l'image "4logo.png") dans une grille.
# L'image finale (la photo seule ou la grille de 4 ou 8) est ensuite envoyée à l'imprimante à l'aide de l'API CUPs.

import picamera
import itertools
import subprocess
import os
from shutil import copyfile
import sys
import time
import logging
import RPi.GPIO as GPIO
from PIL import Image, ImageDraw, ImageFont
import pygame
from pygame.locals import *


IMG1             = "1.jpg"
IMG2             = "2.jpg"
IMG3             = "3.jpg"
IMG4             = "4.jpg"
#IMG4             = "4logo.png"
IMG5             = "5.jpg"
IMG6             = "6.jpg"
IMG7             = "7.jpg"
IMG8             = "8.jpg"
CurrentWorkingDir= "/home/pi/Photomaton_x1x4x8/"
logDir           = "logs"
archiveDir       = "photos"
LARGEUR_ECRAN     = 1280
HAUTEUR_ECRAN    = 960
LARGEUR_PHOTO      = 1280
HAUTEUR_PHOTO     = 960
#LARGEUR_ECRAN     = 1280
#HAUTEUR_ECRAN    = 960
#LARGEUR_PHOTO      = 1920
#HAUTEUR_PHOTO     = 1440
LED_PIN          = 19 #connected to external 12v.
PHOTO_DELAY      = 5 #délai en secondes avant prise de la photo
overlay_renderer = None
buttonEvent      = False
buttonEvent2      = False
buttonEvent3      = False

pygame.init()
#screen = pygame.display.set_mode((0,0),pygame.FULLSCREEN)
screen = pygame.display.set_mode((1920,1080),RESIZABLE)
width, height = screen.get_size()


#on initialise les GPIO en écoute
#Attention au choix des ports ; référez-vous au site https://fr.pinout.xyz/
#J'ai pris les BCM18(GPIO12), BCM24(GPIO18) et BCM26(GPIO37) car les masses sont à côté
GPIO.setmode(GPIO.BCM)
GPIO.setup(18, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(24, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(26, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(LED_PIN, GPIO.OUT)

#Fusionner les 4 images
def convertMergeImages(fileName):
    addPreviewOverlay(150,200,55,"Fusion des images et impression ...")
    #fusionner maintenant toutes les images
    subprocess.call(["montage",
                     IMG1,IMG2,IMG3,IMG4,
                     "-geometry", "+2+2",
                     fileName])
    logging.info("Les 4 photos ont été fusionnées.")

#Modification de l'image solo
def convertMergeImages2(fileName):
    addPreviewOverlay(150,200,55,"Enregistrement et impression ...")
    #Modifier maintenant l'image solo
    subprocess.call(["montage",
                     IMG1,
                     "-geometry", "+2+2",
                     fileName])
    logging.info("Photo transformée.")
    
#Fusionner les 8 images
def convertMergeImages3(fileName):
    addPreviewOverlay(150,200,55,"Fusion des images et impression ...")
    #fusionner maintenant toutes les images
    subprocess.call(["montage",
                     IMG1,IMG2,IMG3,IMG4,IMG5,IMG6,IMG7,IMG8,
                     "-geometry", "+2+2",
                     fileName])
    logging.info("Les 8 photos ont été fusionnées.")
    

#Supprimer les pré-images (celles avant transformation)
def deleteImages(fileName):
    logging.info("Suppression des anciennes images.")
    if os.path.isfile(IMG1):
        os.remove(IMG1)
    if os.path.isfile(IMG2):
        os.remove(IMG2)
    if os.path.isfile(IMG3):
        os.remove(IMG3)
    if os.path.isfile(IMG4):
        os.remove(IMG4)
    if os.path.isfile(IMG5):
        os.remove(IMG5)
    if os.path.isfile(IMG6):
        os.remove(IMG6)
    if os.path.isfile(IMG7):
        os.remove(IMG7)
    if os.path.isfile(IMG8):
        os.remove(IMG8)
    if os.path.isfile(fileName):
        os.remove(fileName);

def cleanUp():
    GPIO.cleanup()
    
def archiveImage(fileName):
    logging.info("Sauvegarde de l'image : "+fileName)
    copyfile(fileName,archiveDir+"/"+fileName)

def countdownFrom(secondsStr):
    secondsNum = int(secondsStr)
    if secondsNum >= 0 :
        while secondsNum > 0 :
            addPreviewOverlay(300,100,240,str(secondsNum))
            time.sleep(1)
            secondsNum=secondsNum-1

def captureImage(imageName):
    addPreviewOverlay(150,200,100,"--> Souriez !   :-)")
    #Sauvegarde de l'image
    camera.capture(imageName, resize=(LARGEUR_PHOTO, HAUTEUR_PHOTO))
    logging.info("Image "+imageName+" enregistrée.")

def addPreviewOverlay(xcoord,ycoord,fontSize,overlayText):
    global overlay_renderer
    img = Image.new("RGB", (LARGEUR_ECRAN, HAUTEUR_ECRAN))
    draw = ImageDraw.Draw(img)
    draw.font = ImageFont.truetype(
                    "/usr/share/fonts/truetype/freefont/FreeSerif.ttf",fontSize)
    draw.text((xcoord,ycoord), overlayText, (255, 20, 147))

    if not overlay_renderer:
        overlay_renderer = camera.add_overlay(img.tobytes(),
                                              layer=3,
                                              size=img.size,
                                              alpha=128);
    else:
        overlay_renderer.update(img.tobytes())

#lancer une action complète pour 4 photos et impression
def play():
    print("Démarrage de la séquence '4 photos'")

    fileName = time.strftime("%Y%m%d-%H%M%S")+".jpg"
    print("Nom de fichier créé : "+fileName)

    #turn on flash
    GPIO.output(LED_PIN,GPIO.HIGH)

    countdownFrom(PHOTO_DELAY)
    captureImage(IMG1)
    time.sleep(1)

    countdownFrom(PHOTO_DELAY)
    captureImage(IMG2)
    time.sleep(1)

    countdownFrom(PHOTO_DELAY)
    captureImage(IMG3)
    time.sleep(1)
    
    countdownFrom(PHOTO_DELAY)
    captureImage(IMG4)
    time.sleep(1)

    #turn off flash
    GPIO.output(LED_PIN,GPIO.LOW)

    convertMergeImages(fileName)
    time.sleep(1)

    archiveImage(fileName)
    deleteImages(fileName)
    
    os.system('lpr -h -PSamsung_Samsung_CLX-3300_Series -#1 -o media=A4 -o scaling=25 "/home/pi/Photomaton_x1x4x8/photos/'+fileName+'"')
    time.sleep(15)
    logging.info("Travail d'impression créé avec succès.")
    print("Travail d'impression créé avec succès.")
    

#lancer une action complète pour 1 photo et impression
def play2():
    print("Démarrage de la séquence '1 photo'")

    fileName = time.strftime("%Y%m%d-%H%M%S")+".jpg"
    print("Nom de fichier créé : "+fileName)

    #turn on flash
    GPIO.output(LED_PIN,GPIO.HIGH)

    countdownFrom(PHOTO_DELAY)
    captureImage(IMG1)
    time.sleep(1)
    
    convertMergeImages2(fileName)
    time.sleep(1)

    archiveImage(fileName)
    deleteImages(fileName)
    
    
    os.system('lpr -h -PSamsung_Samsung_CLX-3300_Series -#1 -o media=A4 -o scaling=25 "/home/pi/Photomaton_x1x4x8/photos/'+fileName+'"')
    time.sleep(15)
    logging.info("Travail d'impression créé avec succès.")
    print("Travail d'impression créé avec succès.")


#lancer une action complète pour 8 photos et impression
def play3():
    print("Démarrage de la séquence '1 photo'")

    fileName = time.strftime("%Y%m%d-%H%M%S")+".jpg"
    print("Nom de fichier créé : "+fileName)

    #turn on flash
    GPIO.output(LED_PIN,GPIO.HIGH)

    countdownFrom(PHOTO_DELAY)
    captureImage(IMG1)
    time.sleep(1)
    
    countdownFrom(PHOTO_DELAY)
    captureImage(IMG2)
    time.sleep(1)
        
    countdownFrom(PHOTO_DELAY)
    captureImage(IMG3)
    time.sleep(1)
        
    countdownFrom(PHOTO_DELAY)
    captureImage(IMG4)
    time.sleep(1)
        
    countdownFrom(PHOTO_DELAY)
    captureImage(IMG5)
    time.sleep(1)
        
    countdownFrom(PHOTO_DELAY)
    captureImage(IMG6)
    time.sleep(1)
        
    countdownFrom(PHOTO_DELAY)
    captureImage(IMG7)
    time.sleep(1)
        
    countdownFrom(PHOTO_DELAY)
    captureImage(IMG8)
    time.sleep(1)
    
    convertMergeImages3(fileName)
    time.sleep(1)

    archiveImage(fileName)
    deleteImages(fileName)
    
    
    os.system('lpr -h -PSamsung_Samsung_CLX-3300_Series -#1 -o media=A4 -o scaling=25 "/home/pi/Photomaton_x1x4x8/photos/'+fileName+'"')
    time.sleep(15)
    logging.info("Travail d'impression créé avec succès.")
    print("Travail d'impression créé avec succès.")
    
def initCamera(camera):
    logging.info("Initialisation de la caméra.")
    #camera settings
    camera.resolution            = (LARGEUR_ECRAN, HAUTEUR_ECRAN)
    camera.framerate             = 24
    camera.sharpness             = 0
    camera.contrast              = 0
    camera.brightness            = 50
    camera.saturation            = 0
    camera.ISO                   = 0
    camera.video_stabilization   = False
    camera.exposure_compensation = 0
    camera.exposure_mode         = 'auto'
    camera.meter_mode            = 'average'
    camera.awb_mode              = 'auto'
    camera.image_effect          = 'none'
    camera.color_effects         = None
    camera.rotation              = 0
    camera.hflip                 = False
    camera.vflip                 = False
    camera.crop                  = (0.0, 0.0, 1.0, 1.0)

def initLogger(output_dir):
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    # create console handler and set level to info
    handler = logging.StreamHandler()
    handler.setLevel(logging.INFO)
    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    # create error file handler and set level to error
    handler = logging.FileHandler(output_dir+"/"+time.strftime("%Y%m%d")+"_error.log","w", encoding=None, delay="true")
    handler.setLevel(logging.ERROR)
    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    # create debug file handler and set level to debug
    handler = logging.FileHandler(output_dir+"/"+time.strftime("%Y%m%d")+"_debug.log","w", encoding=None, delay="true")
    handler.setLevel(logging.DEBUG)
    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)


def onButtonPress():
    logging.info("Demande faite pour 1 image de 4 photos !")
    play()
    #reset the initial welcome message
    addPreviewOverlay(20,200,55,"  --> Rouge=8 photos / Noir=4 photos / Blanc=1 photo")

def onButtonPressBis():
    logging.info("Demande faite pour 1 seule grande photo !")
    play2()
    #reset the initial welcome message
    addPreviewOverlay(20,200,55,"  --> Rouge=8 photos / Noir=4 photos / Blanc=1 photo")

def onButtonPressTer():
    logging.info("Demande faite pour 8 photos !")
    play3()
    #reset the initial welcome message
    addPreviewOverlay(20,200,55,"  --> Rouge=8 photos / Noir=4 photos / Blanc=1 photo")


def onButtonDePress():
    logging.info("Big red button de-pressed!")
    
def onButtonDePressBis():
    logging.info("Big black button de-pressed!")

def onButtonDePressTer():
    logging.info("Big White button de-pressed!")



def AfficherPhoto(fileName): # affiche NomPhoto
    print("loading image: " + fileName)
    background = pygame.image.load(fileName);
    background.convert_alpha()
    background = pygame.transform.scale(background,(width,height))
    screen.blit(background,(0,0),(0,0,width,height))
    pygame.display.flip()




#Flux initial
with picamera.PiCamera() as camera:
    os.chdir(CurrentWorkingDir)

    try:
        initLogger(logDir)
        initCamera(camera)
        GPIO.output(LED_PIN,GPIO.LOW)
        logging.info("Démarrage de l'aperçu")
        camera.start_preview()
        addPreviewOverlay(20,200,55,"--> Rouge=8 photos, Noir=4 photos, Blanc=1 photo")

    

        logging.info("Démarrage de la boucle du programme")
        while True:
            input_state = GPIO.input(24)
            input_state2 = GPIO.input(26)
            input_state3= GPIO.input(18)

            if input_state == False :
                if buttonEvent == False :
                    buttonEvent = True
                    onButtonPress()
            elif buttonEvent == True :
                    buttonEvent = False
                    onButtonDePress()
        
            if input_state2 == False : 
                if buttonEvent2 == False :
                    buttonEvent2 = True
                    onButtonPressBis()
            else :
                if buttonEvent2 == True :
                    buttonEvent2 = False
                    onButtonDePressBis()
      
            if input_state3 == False : 
                if buttonEvent3 == False :
                    buttonEvent3 = True
                    onButtonPressTer()
            else :
                if buttonEvent3 == True :
                    buttonEvent3 = False
                    onButtonDePressTer()
                    
    except BaseException:
        logging.error("Heu ... Exception non gérée : " , exc_info=True)
        camera.close()
        cleanUp()
    finally:
        logging.info("Au revoir ...")
        cleanUp()
        camera.close()

#end
